/*
Stolyarevskiy Sergey, acobw.narod.ru

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation.
	
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
		
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Filter NSLF produces a smoothing of one frame in NN matrix without
the loss of clearness. Influence nearby on the matrix of pixels on
accounting is defined by the crooked dependency, given square-law function.

06.03.2005 V1.0 Beta version.

17.03.2005 V2.0
Processing algorithm eliminated inaccuracy, brought about general reducing brightness on 2.4 %.
Optimization. Increasing a speed (P-IV) on 30%.
Supported by job control, controlled influence level.

27.03.2005 v2.1 Corrected:
-incorrect work after using a scrap of edges

19.04.2005 v2.2 Beta
Processed from the zero a mathematics, more correct attitude to expressing,
panel of adjustment, preview, visualization noise forming, choice of diameter of window of smoothing.

22.04.2005 v2.3 Optimization.

8.11.2005 v2.4
"Turbo" includes separate, independent mode of processing the channels R,G,B.
Supposed application - cartoons.

7.12.2005 v2.5
Increasing a speed for mode matrixes 33

*/

#include <windows.h>
#include <stdio.h>
#include <commctrl.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

int RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);
int InitProc(FilterActivation *fa, const FilterFunctions *ff);

typedef struct MyFilterData {
	IFilterPreview		*ifp;
	int NLevel;
	_int64 NDelit;
	_int64 Mnoj;
	_int64 Drob;
	_int64 SDelit;
	int Matrix;
	int Radius;
	int Noise;
	int Turbo;
} MyFilterData;

void setting(MyFilterData *D);

ScriptFunctionDef _func_defs[]={
    { (ScriptFunctionPtr)ScriptConfig, "Config", "0iii" },
    { NULL },
};

CScriptObject nslf_obj={
    NULL, _func_defs
};

struct FilterDefinition filterDef_ = {
    NULL, NULL, NULL,       // next, prev, module
    "Nonlinear Smoothing Loss-Free",             // name
    "Swabbing an expressing on nonlinear feature without loss of clearness, version 2.5",     // desc
    "Stolyarevskiy Sergey", // maker
    NULL,                   // private_data
    sizeof(MyFilterData),   // inst_data_size
    InitProc,               // initProc
    NULL,                   // deinitProc
    RunProc,				// runProc
	NULL,				    // paramProc
	ConfigProc,				// configProc
    StringProc,				// stringProc
	NULL,					// startProc
	NULL,					// endProc
    &nslf_obj,				// script_obj
    FssProc,				// fssProc
};

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_ = ff->addFilter(fm, &filterDef_, sizeof(FilterDefinition))))
        return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_);
}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
  MyFilterData *mfd = (MyFilterData *)fa->filter_data;
  Pixel32 *src,*src2, *dst;
 _int64 Zero=0x0000000000000000;
 _int64 ND=mfd->NDelit;
 _int64 SD=mfd->SDelit;
 _int64 Drob=mfd->Drob;
 _int64 Mnoj=mfd->Mnoj;
_int64 Nol=0x0000008000800080;
int	th=fa->src.h;
int	tw=fa->src.w;
int wwhh=th*tw;
	src = (Pixel32 *)fa->src.data;
    dst = (Pixel32 *)fa->dst.data;
	src2= (Pixel32 *)fa->src.data;
int i=(fa->src.modulo)>>2;
_asm{			movq mm0, Zero			//
				movq mm7, ND
				movq mm6, Mnoj			//*5
	}

if (mfd->Turbo){
	_asm{
//		psllq mm7, 0
		movq mm1, mm7
		psllq mm1, 16
		por mm7, mm1
		psllq mm1, 16 
		por mm7, mm1
	}
}
int R=mfd->Radius;
//*-*-*-*-*-*-*-*-*-*-*-*-*   *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
if (R>1){
//*-*-*-*-*-*-*-*-*-*-*-*-*  5*5  7*7 *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
for (int yt = 0; yt < th; yt++){
	for (int xt = 0; xt < tw; xt++){
int ys=0;
if (yt>R)ys=yt-R;
int ye=yt+R;
if (ye>=th)ye=th-1;
int Pt=*src++;
_asm {
				movd mm1, Pt			// 
				movq mm5, mm0			// 
				punpcklbw mm1, mm0		//   
}
//***********************************************************************************
if (!mfd->Turbo){
//*********************************   ********************************
do {
int xs=0;
if (xt>R) xs=xt-R;
int xe=xt+R;
if (xe>=tw) xe=tw-1;
register int z=ys*(tw+i);
do {
int Pn=src2[xs+z];
_asm{
movd mm2, Pn			//  
punpcklbw mm2, mm0		//   
psubsw mm2, mm1			//n-T
movq mm3, mm2			//   n-T
psrlw mm3, 1			//  2
pmullw mm3, mm3			//(R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//  2
movq mm4, mm7			//  128
psubusw mm4, mm3		//128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}										//4,3,
//		}
		}while(xs++<xe);
}while(ys++<ye);
//***********************************************************************************
	}else{
//******************************************* TURBO *********************************
do {
int xs=0;
if (xt>R) xs=xt-R;
int xe=xt+R;
if (xe>=tw) xe=tw-1;
register int z=ys*(tw+i);
do {
int Pn=src2[xs+z];
		_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
			}
//		}
	}while(xs++<xe);
}while(ys++<ye);
	}

int T;
_asm{
		movq mm2, SD	
		psraw mm5, mm2
		pmullw mm5, mm6
		movq mm2, Drob
		psraw mm5, mm2			//cor=cor/128       7  4
		}
if (mfd->Noise)
_asm{	movq mm1, Nol
		psllq mm5, 4}   //*****
_asm{
		paddsw mm1, mm5			//+r
		packuswb mm1, mm0		//
		movd T, mm1
		}
*dst++=T;	
	}
        src = (Pixel32 *)((char *)src + fa->src.modulo);
        dst = (Pixel32 *)((char *)dst + fa->dst.modulo);
	}

//***********************************************************************************
	}else{
//*-*-*-*-*-*-*-*-*-*-*-*-*  3*3 *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
int k2=(tw+i)*(th-1);
memcpy (dst+k2,src+k2,tw*4);
memcpy (dst,src,tw*4);
for (int yt = 1; yt < th-1; yt++){
	dst[yt*(tw+i)]=src[yt*(tw+i)];
	dst[yt*(tw+i)+tw-1]=src[yt*(tw+i)+tw-1];
	for (int xt = 1; xt < tw-1; xt++){
int Pt=src[yt*(tw+i)+xt];				// & 0x00FFFFFF;
_asm {
				movd mm1, Pt			// 
				movq mm5, mm0			// 
				punpcklbw mm1, mm0		//   
}
//***********************************************************************************
if (!mfd->Turbo){
//*********************************   ********************************
//(-1,-1)
int Pn=src[(yt-1)*(tw+i)+xt-1];
_asm{
movd mm2, Pn			//  
punpcklbw mm2, mm0		//   
psubsw mm2, mm1			//n-T
movq mm3, mm2			//   n-T
psrlw mm3, 1			//  2
pmullw mm3, mm3			//(R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//  2
movq mm4, mm7			//  128
psubusw mm4, mm3		//128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}
//(-1,0)
Pn=src[(yt-1)*(tw+i)+xt];
_asm{
movd mm2, Pn			//  
punpcklbw mm2, mm0		//   
psubsw mm2, mm1			//n-T
movq mm3, mm2			//   n-T
psrlw mm3, 1			//  2
pmullw mm3, mm3			//(R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//  2
movq mm4, mm7			//  128
psubusw mm4, mm3		//128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}
//(-1,+1)
Pn=src[(yt-1)*(tw+i)+xt+1];
_asm{
movd mm2, Pn			//  
punpcklbw mm2, mm0		//   
psubsw mm2, mm1			//n-T
movq mm3, mm2			//   n-T
psrlw mm3, 1			//  2
pmullw mm3, mm3			//(R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//  2
movq mm4, mm7			//  128
psubusw mm4, mm3		//128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}
//(0,-1)
Pn=src[(yt)*(tw+i)+xt-1];
_asm{
movd mm2, Pn			//   
punpcklbw mm2, mm0		//    
psubsw mm2, mm1			// n-T
movq mm3, mm2			//    n-T
psrlw mm3, 1			//   2
pmullw mm3, mm3			// (R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//   2
movq mm4, mm7			//  128
psubusw mm4, mm3		// 128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}
//(0,+1)
Pn=src[(yt)*(tw+i)+xt+1];
_asm{
movd mm2, Pn			//  
punpcklbw mm2, mm0		//   
psubsw mm2, mm1			//n-T
movq mm3, mm2			//   n-T
psrlw mm3, 1			//  2
pmullw mm3, mm3			//(R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//  2
movq mm4, mm7			//  128
psubusw mm4, mm3		//128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}
//(1,-1)
Pn=src[(yt+1)*(tw+i)+xt-1];
_asm{
movd mm2, Pn			//  
punpcklbw mm2, mm0		//   
psubsw mm2, mm1			//n-T
movq mm3, mm2			//   n-T
psrlw mm3, 1			//  2
pmullw mm3, mm3			//(R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//  2
movq mm4, mm7			//  128
psubusw mm4, mm3		//128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}
//(1,0)
Pn=src[(yt+1)*(tw+i)+xt];
_asm{
movd mm2, Pn			//  
punpcklbw mm2, mm0		//   
psubsw mm2, mm1			//n-T
movq mm3, mm2			//   n-T
psrlw mm3, 1			//  2
pmullw mm3, mm3			//(R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//  2
movq mm4, mm7			//  128
psubusw mm4, mm3		//128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}
//(+1,+1)
Pn=src[(yt+1)*(tw+i)+xt+1];
_asm{
movd mm2, Pn			//  
punpcklbw mm2, mm0		//   
psubsw mm2, mm1			//n-T
movq mm3, mm2			//   n-T
psrlw mm3, 1			//  2
pmullw mm3, mm3			//(R^2 G^2 B^2)
movq mm4, mm3			//   
psrlq mm4, 16			//  G
paddusw mm3, mm4		// G+B
psrlq mm4, 16			//  R
paddusw mm3, mm4		// R+(G+B)
psraw mm3, 2			//  2
movq mm4, mm7			//  128
psubusw mm4, mm3		//128- modylm 
punpcklwd mm4, mm4
punpckldq mm4, mm4
pmullw mm2, mm4
paddsw mm5, mm2
}

//***********************************************************************************
	}else{
//******************************************* TURBO *********************************

//(-1,-1)
int Pn=src[(yt-1)*(tw+i)+xt-1];
_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
}
//(-1,0)
Pn=src[(yt-1)*(tw+i)+xt];
_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
}
//(-1,+1)
Pn=src[(yt-1)*(tw+i)+xt+1];
_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
}
//(0,-1)
Pn=src[(yt)*(tw+i)+xt-1];
_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
}
//(0,+1)
Pn=src[(yt)*(tw+i)+xt+1];
_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
}
//(1,-1)
Pn=src[(yt+1)*(tw+i)+xt-1];
_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
}
//(1,0)
Pn=src[(yt+1)*(tw+i)+xt];
_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
}
//(+1,+1)
Pn=src[(yt+1)*(tw+i)+xt+1];
_asm{
				movd mm2, Pn			//  
				punpcklbw mm2, mm0		//   
				psubsw mm2, mm1			//n-T
				movq mm3, mm2			//   n-T
				psrlw mm2, 1			//  2
				pmullw mm2, mm2			//   ((Tn-T)/4)^2
				movq mm4, mm7			// 64
	//			psllw mm4, 2
				psubusw mm4, mm2		//64-((Tn-T)/2)^2
				pmullw mm3, mm4			//(n-T)*(64-((Tn-T)/2)^2)
				paddsw mm5, mm3			//cor=cor+(n-T)*(16-((Tn-T)/4)^2)
}
//---		
	}
int T;
_asm{
		movq mm2, SD	
		psraw mm5, mm2
		pmullw mm5, mm6
		movq mm2, Drob
		psraw mm5, mm2			//cor=cor/128       7  4
		}
if (mfd->Noise)
_asm{	movq mm1, Nol
		psllq mm5, 4}   //*****
_asm{
		paddsw mm1, mm5			//+r
		packuswb mm1, mm0		//
		movd T, mm1
		}
dst[yt*(tw+i)+xt]=T;	
	}
	}
}
	//*-*-*-*-*-*-*-*-*-*-*-*-*   *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
_asm{emms}
return 0;
}

BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

    switch(msg) {
        case WM_INITDIALOG:
            SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;
            SendDlgItemMessage (hdlg, IDC_SLIDER_LEVEL, TBM_SETRANGE, false, MAKELONG (0, 4));
            SendDlgItemMessage (hdlg, IDC_SLIDER_LEVEL, TBM_SETPOS, true, mfd->NLevel);
            SendDlgItemMessage (hdlg, IDC_SLIDER_MATRIX, TBM_SETRANGE, false, MAKELONG (0, 2));
			SendDlgItemMessage (hdlg, IDC_SLIDER_MATRIX, TBM_SETPOS, true, mfd->Matrix);
			CheckDlgButton(hdlg, IDC_NOISE, mfd->Noise?BST_CHECKED:BST_UNCHECKED);
			CheckDlgButton(hdlg, IDC_TURBO, mfd->Turbo?BST_CHECKED:BST_UNCHECKED);
		    mfd->ifp->InitButton(GetDlgItem(hdlg, IDPREVIEW));
            return TRUE;

	case WM_HSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_LEVEL))
			{
				mfd->NLevel=SendDlgItemMessage (hdlg, IDC_SLIDER_LEVEL, TBM_GETPOS, 0, 0);
				setting(mfd);		
				mfd->ifp->RedoFrame();
			}
			else if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_MATRIX))
			{
				mfd->Matrix=SendDlgItemMessage (hdlg, IDC_SLIDER_MATRIX, TBM_GETPOS, 0, 0);
				setting(mfd);
				mfd->ifp->RedoFrame();
			}
			break;
     case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
				mfd->Noise= !!IsDlgButtonChecked(hdlg, IDC_NOISE);
				mfd->NLevel=SendDlgItemMessage (hdlg, IDC_SLIDER_LEVEL, TBM_GETPOS, 0, 0);
				mfd->Matrix=SendDlgItemMessage (hdlg, IDC_SLIDER_MATRIX, TBM_GETPOS, 0, 0);
                EndDialog(hdlg, 0);
			setting(mfd);
               return TRUE;
			case IDPREVIEW:
				setting(mfd);
				mfd->ifp->Toggle(hdlg);
				mfd->ifp->RedoFrame();
				break;
			case IDC_NOISE:
			mfd->Noise= !!IsDlgButtonChecked(hdlg, IDC_NOISE);
			setting(mfd);
			mfd->ifp->RedoFrame();
			case IDC_TURBO:
			mfd->Turbo= !!IsDlgButtonChecked(hdlg, IDC_TURBO);
			setting(mfd);
			mfd->ifp->RedoFrame();
            return TRUE;
            case IDCANCEL:
                EndDialog(hdlg, 1);
                return FALSE;
            }
            break;
    }
    return FALSE;
}

int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;	
	mfd->ifp = fa->ifp;
	return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER), hwnd,
            ConfigDlgProc, (LPARAM)fa->filter_data);
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
    const char *modesS[5]={
        "minimum",
        "weakly",
        "normal",
		"powerfully",
		"maximum"
	};
    const char *modesR[3]={
        "33",
        "55",
        "77"
	};
    const char *modesT[2]={
        "",
        ", turbo"
     };
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	sprintf(str, " ( %s, %s %s )",modesS[mfd->NLevel],modesR[mfd->Matrix],modesT[mfd->Turbo]);
	}

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    mfd->NLevel=argv[0].asInt();
	mfd->Matrix=argv[1].asInt();
	mfd->Turbo=argv[2].asInt();
	setting(mfd);
}

bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
      MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	  _snprintf(buf, buflen, "Config(%d, %d, %d)", mfd->NLevel, mfd->Matrix, mfd->Turbo);
    return true;
}

int InitProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    mfd->NLevel=2;
	mfd->Matrix=1;
    return 0;
}

void setting(MyFilterData *D){
  	switch (D->NLevel){
case 0:
	D->NDelit=0x0000000000000008;
	D->SDelit=0x0000000000000003; //16
	break;
case 1:
	D->NDelit=0x0000000000000010;
	D->SDelit=0x0000000000000004; //32
	break;
case 2:
	D->NDelit=0x0000000000000020;
	D->SDelit=0x0000000000000005; //64
	break;
case 3:
	D->NDelit=0x0000000000000040;
	D->SDelit=0x0000000000000006; //128
	break;
case 4:
	D->NDelit=0x0000000000000080;
	D->SDelit=0x0000000000000007; //256
	break;
}

	switch (D->Matrix){
case 0:
	D->Drob=0x0000000000000006; //128
	D->Mnoj=0x0000000700070007;
	D->Radius=1;
	break;
case 1:
	D->Drob=0x0000000000000007;  //128
	D->Mnoj=0x0000000500050005;
	D->Radius=2;
	break;
case 2:
	D->Drob=0x000000000000000A;  //256
	D->Mnoj=0x0000001500150015;
	D->Radius=3;
	break;
}
}
